#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PIL import Image
from super_image import EdsrModel, ImageLoader


def superscale(img, model=EdsrModel, scale=2):
    mod = model.from_pretrained('eugenesiow/edsr-base', scale=scale)
    inputs = ImageLoader.load_image(Image.fromarray(img.astype('uint8'), 'RGB'))
    preds = mod(inputs)
    return preds


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import colorsys
import colorir as cir
from colour import Color
import ArtSciColor.constants as cst

def sortSwatchByFrequency(freqSwatch):
    freqSwatch.sort(key=lambda rgb: colorsys.rgb_to_hsv(*rgb[0].get_rgb()))
    swatchHex = [s[0] for s in freqSwatch]
    return swatchHex


def sortSwatchHSV(
        freqSwatch,
        hue_classes=None, gray_thresh=255, 
        gray_start=True, alt_lum=True, invert_lum=False
    ):
    swatchHex = [s[0].hex for s in freqSwatch]
    swatchHex.sort(
        key=cir.hue_sort_key(
            hue_classes=hue_classes, gray_thresh=gray_thresh,
            gray_start=gray_start, alt_lum=alt_lum, invert_lum=invert_lum
        )
    )
    swatchHex = [Color(c) for c in swatchHex]
    return swatchHex


def getTextColor(hexBackground, threshold=0.55):
    # https://stackoverflow.com/questions/3942878/how-to-decide-font-color-in-white-or-black-depending-on-background-color
    (r, g, b) = hexBackground.rgb
    tcol = (0, 0, 0) if (r*0.299+g*0.587+b*0.114)>threshold else (1, 1, 1)
    return Color(rgb=tcol)


def getSwatch(swatchID):
    return cst.SWATCHES[swatchID]


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .paths import *
from .image import *
from .color import *
from .swatches import *
from .auxiliary import *
from .constants import *
from .fingerprint import *
from .fingerprintWrappers import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# https://matplotlib.org/stable/api/_as_gen/matplotlib.colors.ListedColormap.html

from colour import Color
from numpy import interp
from matplotlib.colors import LogNorm, PowerNorm, SymLogNorm
from matplotlib.colors import ColorConverter, LinearSegmentedColormap

def addHexOpacity(colors, alpha='1A'):
    return [c+alpha for c in colors]


def replaceHexOpacity(colors, alpha='FF'):
    return [i[:-2]+alpha for i in colors]


def generateAlphaColorMapFromColor(color, minAlpha=0, maxAlpha=1):
    """Creates a linear cmap that runs from transparent to full opacity of the provided color.

    Args:
        color (hex string or rgb tuple): Color for cmap
        minAlpha (float): Minimum transparency
        maxAlpha (float): Maximum opacity

    Returns:
        cmap: Matplotlib colormap
    """
    if (type(color) is str):
        color = list(Color(color).rgb)
    alphaMap = LinearSegmentedColormap.from_list(
        'alphaMap', 
        [
            (0, 0, 0, minAlpha), 
            (color[0], color[1], color[2], maxAlpha)
        ]
    )
    return alphaMap

def colorPaletteFromHexList(clist):
    """Takes a list of colors in hex-form and generates a matplotlib-compatible
    cmap function.

    Args:
        clist (list of hex colors): List of colors to be used in the color palette in linear form.

    Returns:
        cmap: LinearSegmentedColormap function for color maps.
    """    
    c = ColorConverter().to_rgb
    clrs = [c(i) for i in clist]
    rvb = LinearSegmentedColormap.from_list("hexMap", clrs)
    return rvb


###############################################################################
# Generate cmap normalizing functions
###############################################################################
def normLinear(vmin=0, vmax=1):
    """Generates a linear interpolation function for cmaps (range 0 to 1) in the
    vmin to vmax range.

    Args:
        vmin (float): Minimum possible input value.
        vmax (float): Maximum possible input value.

    Returns:
        function: Normalizing function to be evaluated.
    """    
    linInterp = lambda x: interp(x, [vmin, vmax], [0, 1], left=None)
    return linInterp

def evalNormColor(
        x, cmap,  
        normFun=SymLogNorm(1, vmin=0, vmax=10),
    ):
    """Evaluates a cmap on the x value given a normalizing function to scale 
    this input value.

    Args:
        x (float): Value to evaluate on.
        cmap (colormap): Matplotlib-compatible colormap.
        normFun (function): Normalizing function (normLinear, LogNorm, PowerNorm, SymLogNorm, etc).

    Returns:
        color: Evaluated color on the cmap.
    """    
    scaledValue = normFun(x)
    color = cmap(scaledValue)
    return color


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import warnings
import pandas as pd
import pkg_resources
from os.path import join
import ArtSciColor.paths as pth
import ArtSciColor.auxiliary as aux

SWATCH_DIMS = {'width': 750, 'height': 50}
NOT_ART = set(('Splatoon', 'Ghibli'))
CATEGORIES = {
    'Art': set((
        "Magritte",
        "Matisse", "Nolde", "Warhol", "Monet", "Kirchner", 
        "Miro", "Picasso", "Kandinsky", "EdnaAndrade", "Signac",
        "VanGogh", "DarbyBannard", "UmbertoBoccioni"
    )),
    'Movies': set(('Ghibli', 'Disney')),
    'Gaming': set(('Splatoon', )),
    'Other': set(('chipdelmal', 'coolors', 'lospec', 'Public'))
}
ARTISTS_SET = set.union(*list(CATEGORIES.values()))
DF_SORTING = (
    'artist', 'title', 'palette', 'hash', 
    'clusters', 'clustering', 'url', 'filename'
)
###############################################################################
# Load Serialized data
#   https://stackoverflow.com/questions/779495/access-data-in-package-subdirectory
#   https://packaging.python.org/en/latest/guides/distributing-packages-using-setuptools/
###############################################################################
DATA_PATH = pkg_resources.resource_filename('ArtSciColor', 'data/')
(PT_SW, PT_DF) = (join(DATA_PATH, 'SWATCHES.bz'), join(DATA_PATH, 'DB.csv'))
try:
    SWATCHES = aux.loadDatabase(PT_SW, df=False)
    SWATCHES_DF = pd.read_csv(PT_DF)
except:
    warnings.warn("Missing colors database!")
###############################################################################
# HTML/MD Text
###############################################################################
# hdr = ('ID', 'Palette', 'Hex Palette')
hdr = ('Title', 'ID', 'Palette', 'Hex Palette')
# ('Artist', 'Title', 'Palette', 'ID') # , 'Hex Palette')
RDM_HEADER = [
    f'<th style="text-align: center; vertical-align: middle;">{e}</th>'
    for e in hdr
]
RDM_TEXT = '''
<!DOCTYPE html>
<html><body>
<h1>{}</h1>
<p>Click on the color palette to see the original artwork or source!</p>
<table style="width:100%">
<tr>{}</tr>{}
</table>
</body></html>
'''
def generateHTMLEntry(artist, url, title, relPth, hname, strPal):
    entry = [
        f'<td style="text-align: center; vertical-align: middle;">{e}</td>'
        for e in (
            # f'<p style="font-size:14px">{artist}</p>', # Artist
            f'<a href={url} style="font-size:14px">{title}</a>', # Title 
            f'<p style="font-size:14px">{hname}</p>', # Hash Name
            f'<a href={url} style="font-size:14px"><img style="border-radius: 14px;" src="{relPth}" height="25"></a>', # Relative Path 
            f'<p style="font-size:14px">{strPal}</p>' # String Palette
        )
    ]
    mdRow = '\r<tr>'+' '.join(entry)+'</tr>'
    return mdRow


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
from PIL import Image
from sklearn.cluster import KMeans
import ArtSciColor.image as amg
import ArtSciColor.swatches as swt
import ArtSciColor.fingerprint as fng
import warnings
warnings.filterwarnings("ignore")


def getSwatchedImage(
        img, maxSide=150,
        cFun=KMeans, cArgs={'n_clusters': 10, 'max_iter': 1000, 'n_init': 10},
        grpFun=np.median, round=True, 
        HSVSort=False, hueClasses=10, grayThreshold=25,
        barHeight=0.15, barProportional=True,
        font='Arial', fontSize=50
    ):
    resized = amg.resizeCV2BySide(img, maxSide)
    (pixels, labels, model) = fng.calcDominantColors(
        resized, cFun=cFun, cArgs=cArgs
    )
    swatch = fng.getDominantSwatch(pixels, labels, grpFun=grpFun, round=round)
    swatchHex = (
        swt.sortSwatchHSV(swatch, hue_classes=hueClasses, gray_thresh=grayThreshold)
        if HSVSort else
        swt.sortSwatchByFrequency(swatch)
    )
    bars = fng.genColorSwatchFromImg(
        img, barHeight, swatchHex, 
        proportionalHeight=barProportional
    )
    barsImg = fng.addHexColorText(
        Image.fromarray(bars.astype('uint8'), 'RGB'), 
        swatchHex, font=font, fontSize=fontSize
    )
    newIMG = np.row_stack([img, barsImg])
    imgOut = Image.fromarray(newIMG.astype('uint8'), 'RGB')
    return {'image': imgOut, 'swatch': swatchHex, 'model': model}


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import cv2

def readCV2Image(
        imgPath, scaleFactor=1, interpolation=cv2.INTER_AREA
    ):
    bgr = cv2.imread(imgPath)
    img = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
    return img

def resizeCV2Image(
        img, resizeToFraction, 
        interpolation=cv2.INTER_AREA
    ):
    # Rescale image -----------------------------------------------------------
    (width, height) = (
        int(img.shape[1]*resizeToFraction), 
        int(img.shape[0]*resizeToFraction)
    )
    dim = (width, height)
    resized = cv2.resize(img, dim, interpolation=interpolation)
    return resized

def resizeCV2ImageAspect(image, width=None, height=None, inter=cv2.INTER_AREA):
    # initialize the dimensions of the image to be resized and
    # grab the image size
    dim = None
    (h, w) = image.shape[:2]

    # if both the width and height are None, then return the
    # original image
    if width is None and height is None:
        return image

    # check to see if the width is None
    if width is None:
        # calculate the ratio of the height and construct the
        # dimensions
        r = height / float(h)
        dim = (int(w * r), height)

    # otherwise, the height is None
    else:
        # calculate the ratio of the width and construct the
        # dimensions
        r = width / float(w)
        dim = (width, int(h * r))

    # resize the image
    resized = cv2.resize(image, dim, interpolation = inter)

    # return the resized image
    return resized

def resizeCV2BySide(img, maxSize):
    if img.shape[0] > img.shape[1]:
        resized = resizeCV2ImageAspect(img, width=maxSize)
    else:
        resized = resizeCV2ImageAspect(img, height=maxSize)
    return resized




import os
import setuptools
from version import version as this_version


this_directory =  os.path.abspath(os.path.dirname(__file__))
version_path = os.path.join(this_directory, 'ArtSciColor', '_version.py')
with open(version_path, 'wt') as fversion:
    fversion.write('__version__ = "'+this_version+'"')


REQUIRED_PACKAGES=[
    'matplotlib>=3.3.2', 'colour>=0.1.5', 'colorir', 
    'Pillow', 'opencv-python', 'super-image',
    'scikit-learn', 'numpy', 'pandas',
    'compress-pickle'
]


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="ArtSciColor",                                                
    install_requires=REQUIRED_PACKAGES,                         
    version=this_version,                                      
    author="chipdelmal",                                        
    scripts=[],                                                 
    author_email="chipdelmal@gmail.com",                        
    description="Color palettes for scientific purposes",       
    long_description=long_description,                          
    long_description_content_type="text/markdown",              
    url="https://github.com/Chipdelmal/ArtSciColor",                   
    packages=setuptools.find_packages(),                        
    classifiers=[                                               
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    package_data={
        'ArtSciColor': [
            './data/DB.bz', 
            './data/DB.csv',
            './data/SWATCHES.bz'
        ],
    },
    python_requires='>=3.10',
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import csv
import cv2
import numpy as np
from PIL import Image
from colour import Color
from collections import Counter
from sklearn.cluster import KMeans
from PIL import Image, ImageDraw, ImageFont
import ArtSciColor.swatches as sws
import ArtSciColor.auxiliary as aux


def rgbToHex(rgb):
    '''
    Converts an RGB triplet to its hex equivalent.
    * I: RGB 8-bit list
    * O: Hex color string
    '''
    return '#%02x%02x%02x' % (int(rgb[0]), int(rgb[1]), int(rgb[2]))

def hexToRgb(chex, eightBit=False):
    if eightBit:
        return tuple([int(i*255) for i in chex.rgb])
    else:
        return chex.rgb

def calcDominantColors(
        img, 
        cFun=KMeans, cArgs={'n_clusters': 10, 'max_iter': 1000}
    ):
    '''
    Returns a tuple with the dominant colors arrays and their
        clustering labels (sklearn).
    * I: 
        -img: Image imported with cv2.imread
        -clstNumb: Number of desired clusters
        -maxIter: Maximum iterations number for detection
    * O: Dominant colors arrays and clusters labels (sklearn)
    '''
    frame = img.reshape((img.shape[0]*img.shape[1], 3))
    # Cluster the colors for dominance detection
    kmeans = cFun(**cArgs).fit(frame)
    labels = kmeans.labels_
    return (frame, labels, kmeans)


def colorClusterCentroid(cluster, cFun=np.median, round=True):
    cntr = [cFun(cel) for cel in cluster.T]
    if round:
        return [int(c) for c in cntr]
    else:
        return cntr


def getDominantSwatch(
        pixels, labels, 
        grpFun=np.median, round=True
    ):
    cntLbls = [i[0] for i in Counter(labels).most_common()]
    clusters = [pixels[labels==i] for i in cntLbls]
    swatch = [
        (
            Color(rgbToHex(colorClusterCentroid(c, cFun=grpFun, round=round))), 
            c.shape[0]
        ) 
        for c in clusters
    ]
    return swatch


def reshapeColor(colorEightBit):
    '''
    Returns a color triplet in the range of 0 to 1 from 0 to 255.
    * I: List of 8-bit RGB components
    * O: Normalized RBG components
    '''
    return [i / 255 for i in colorEightBit]


def upscaleColor(colorNormalized):
    '''
    Returns a color triplet of 0 to 255 from 0 to 1.
    * I: Normalized list of RGB components
    * O: List of 8-bit RGB components
    '''
    return [int(round(i * 255)) for i in colorNormalized]



def calcHexAndRGBFromPalette(palette):
    '''
    Returns the hex and RGB codes for the colors in a palette.
    * I: Color palette
    * O: Dictionary with hex and rgb colors
    '''
    sortedPalette = [upscaleColor(i) for i in palette]
    (hexColors, rgbColors) = (
            [rgbToHex(i) for i in sortedPalette],
            sortedPalette
        )
    return {'hex': hexColors, 'rgb': rgbColors}


def genColorSwatch(width, height, swatch):
    '''
    Creates a color swatch that is proportional in height to the original
       image (whilst being the same width).
    * I:
        -img: Image imported with cv2.imread
        -heightProp: Desired height of the swatch in proportion to original img
        -palette: Calculated palette through dominance detection
    * O:
    '''
    palette = [hexToRgb(c) for c in swatch]
    clstNumber = len(palette)
    pltAppend = np.zeros((height, width, 3))
    (wBlk, hBlk) = (round(width/clstNumber), height)
    for row in range(hBlk):
        colorIter = -1
        for col in range(width):
            if (col%wBlk==0) and (colorIter<clstNumber-1):
                colorIter=colorIter+1
            pltAppend[row][col] = palette[colorIter]
    return pltAppend*255


def genColorSwatchFromImg(img, barsHeight, swatch, proportionalHeight=True):
    '''
    Creates a color swatch that is proportional in height to the original
       image (whilst being the same width).
    * I:
        -img: Image imported with cv2.imread
        -heightProp: Desired height of the swatch in proportion to original img
        -palette: Calculated palette through dominance detection
    * O:
    '''
    palette = [hexToRgb(c) for c in swatch]
    clstNumber = len(palette)
    (height, width, depth) = img.shape
    if proportionalHeight:
        pltAppend = np.zeros((round(height*barsHeight), width, depth))
        (wBlk, hBlk) = (round(width/clstNumber), round(height*barsHeight))
    else:
        pltAppend = np.zeros((barsHeight, width, depth))
        (wBlk, hBlk) = (round(width/clstNumber), barsHeight)
    for row in range(hBlk):
        colorIter = -1
        for col in range(width):
            if (col%wBlk==0) and (colorIter<clstNumber-1):
                colorIter=colorIter+1
            pltAppend[row][col] = palette[colorIter]
    return pltAppend*255


def writeColorPalette(filepath, palette):
    '''
    Exports the HEX and RGB values of the palette to a tsv file.
    * I:
        -filepath: Path on disk to write the file to
        -palette: Palette output of the getDominancePalette
    * O:
        -True
    '''
    with open(filepath, 'w') as csvfile:
        wtr = csv.writer(csvfile, delimiter='\t')
        for i in range(len(palette['hex'])):
            wtr.writerow([
                    palette['hex'][i],
                    palette['rgb'][i]
                ])
    return True


def genColorBar(width, height, color=[0, 0, 0], depth=3):
    '''
    Creates a solid color bar to act as visual buffer between frames rows.
    * I:
        -width: Desired width of the bar
        -height: Desired height of the bar
        -color: Desired color of the bar
        -depth: Number of color components (3 for RGB)
    * O:
        -colorBar: Image with a solid color bar
    '''
    colorBar = np.full((height, width, depth), color)
    return colorBar


def getDominancePalette(
            imgPath,
            clstNum=6,
            maxIters=1000,
            colorBarHeight=.03,
            bufferHeight=.005,
            colorBuffer=[0, 0, 0]
        ):
    '''
    Wrapper function that puts together all the elements to create the frame
        with its swatch, and buffer bars.
    * I:
        -imgPath: Image location
        -clstNum: Number of desired clusters
        -maxIters: Maximum number of iterations for convergence
        -colorBarHeight: Height of the color swatch (as proportion to img)
        -bufferHeight: Height of the color buffer (as proportion to img)
        -colorBuffer: Color of the buffer
    * O:
        -imgOut: Compiled image with swatch and buffers
        -swatch: Color swatch
        -palette: Hex and RGB color values
    '''
    # Load image --------------------------------------------------------------
    bgr = cv2.imread(imgPath)
    img = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
    (height, width, depth) = img.shape
    # Cluster for dominance ---------------------------------------------------
    (colors, labels) = calcDominantColors(
            img, cltsNumb=clstNum, maxIter=maxIters
        )
    (hexColors, rgbColors) = calcHexAndRGBFromPalette(colors)
    # Create color swatch -----------------------------------------------------
    colorsBars = genColorSwatch(img, colorBarHeight, colors)
    # Put the image back together ---------------------------------------------
    whiteBar = genColorBar(
            width, round(height * bufferHeight), color=colorBuffer
        )
    newImg = np.row_stack((
            whiteBar, colorsBars,
            img,
            colorsBars, whiteBar
        ))
    palette = calcHexAndRGBFromPalette(colors)
    swatch = Image.fromarray(colorsBars.astype('uint8'), 'RGB')
    imgOut = Image.fromarray(newImg.astype('uint8'), 'RGB')
    return (imgOut, swatch, palette)



def addHexColorText(
        barsImg, swatchHex, 
        font='Avenir', fontSize=75, hexLabel=True
    ):
    font = ImageFont.truetype(aux.getFontFile(family=font), fontSize)
    draw = ImageDraw.Draw(barsImg)
    (W, H) = (barsImg.width/len(swatchHex), barsImg.height/2)
    for (ix, hex) in enumerate(swatchHex):
        (colorHex, colorRGB) = (hex.hex.upper(), hex.rgb)
        tcol = sws.getTextColor(hex)
        label = (
            colorHex 
            if hexLabel else 
            str(tuple([int(255*i) for i in colorRGB]))
        )
        # Generate bbox and draw centered -------------------------------------
        bbox = draw.textbbox(xy=(0, 0), text=label, font=font, align='center')
        (w, h) = (bbox[2]-bbox[0], bbox[3]-bbox[1])
        xy = (((2*ix+1)*W-w)/2, H-h/2)
        draw.text(
            xy, label, 
            tuple([int(255*i) for i in tcol.rgb]), 
            font=font, align='center'
        )
    return barsImg


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pandas as pd
from hashlib import sha256
from matplotlib import font_manager
from compress_pickle import dump, load

###############################################################################
# Folders, Notebooks and Fonts
###############################################################################
def makeFolder(path):
    """Crates a folder in the specified directory.

    Args:
        path (string): Path of the folder than needs to be created.

    """
    if not os.path.exists(path):
        try:
            os.mkdir(path)
        except OSError:
            raise OSError(
                    "Can't create destination directory (%s)!" % (path)
                )
            
def isNotebook():
    """Checks if the script is running from a Jupyter environment.

    Returns:
        bool: Flags Jupyter environment. 
    """
    try:
        shell = get_ipython().__class__.__name__
        if shell == 'ZMQInteractiveShell':
            return True   # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False    

def getFontFile(family, weight='regular'):
    font = font_manager.FontProperties(family=family, weight=weight)
    file = font_manager.findfont(font)
    return file

###############################################################################
# Color Palette related functions
###############################################################################
def hashFilename(string, length=20):
    # https://stackoverflow.com/questions/14023350/cheap-mapping-of-string-to-small-fixed-length-string
    string = string.encode('utf-8')
    if length<len(sha256(string).hexdigest()):
        return sha256(string).hexdigest()[:length].upper()
    else:
        x = str(len(sha256(string).hexdigest()))
        exStr = f"Length too long. Length of {length} when hash length is {x}."
        raise Exception(exStr)

def loadDatabase(DBPath, df=True):
    exists = os.path.isfile(DBPath)
    if df:
        db = load(DBPath) if exists else pd.DataFrame()
    else:
        db = load(DBPath) if exists else dict()
    return db
        
def dumpDatabase(database, DBPath):
    dump(database, DBPath)
    return True

def exportDatabase(database, DBPath):
    database.to_csv(DBPath, index=False)
      
def listPalToStr(listPal):
    return str(listPal).replace('[','').replace(']','').replace("'",'')